@@ -49,7 +49,6 @@ public class Main
   public Main () throws Exception
   {
     String classname = "org.eclipse.jdt.internal.compiler.batch.Main";
-    String libname = "lib-org-eclipse-jdt.so";
     Class klass = null;
     try
       {
@@ -57,22 +56,11 @@ public class Main
       }
     catch (ClassNotFoundException e)
       {
-	ClassLoader loader;
-        File libFile = new File (Config.ECJLIB_INST_DIR + "/" + libname);
-
-	if (!libFile.exists () || !libFile.canRead ())
-	  {
-	      File jar = new File (Config.JAR_INST_DIR + "/jdtcore.jar");
-	      if (!jar.exists () || !jar.canRead ())
-		  throw e;
-	      loader = new URLClassLoader(new URL[] {jar.toURL ()});
-	  }
-	else
-	  {
-	      loader = new URLClassLoader(new URL[] {
-		new URL("gcjlib://" + Config.ECJLIB_INST_DIR + "/" + libname)});
-	  }
+	File jar = new File (Config.JAR_INST_DIR + "/jdtcore.jar");
+	if (!jar.exists () || !jar.canRead ())
+	  throw e;
 
+	ClassLoader loader = new URLClassLoader(new URL[] {jar.toURL ()});
 	try
 	  {
 	    klass = loader.loadClass (classname);
@@ -88,6 +76,27 @@ public class Main
     ecjMethod = klass.getMethod ("compile", new Class[] {String[].class});
   }
     
+  // Expand a extensions directory list into a classpath.
+  private String expandExtDirs (String extdirs)
+  {
+    StringBuffer paths = new StringBuffer("");
+    String[] dirs = extdirs.split (File.pathSeparator);
+    for (int i = 0; i < dirs.length; i++)
+      {
+	File dir = new File (dirs[i]);
+	if (dir.isDirectory ())
+	  {
+	    File[] files = dir.listFiles ();
+	    for (int j = 0; j < files.length; j++)
+	      {
+		paths.append (File.pathSeparator);
+		paths.append (files[j].getAbsolutePath());
+	      }
+	  }
+      }
+    return paths.toString ();
+  }
+
   public int compile (String[] args) throws Exception
   {
     // FIXME: Remove arguments supported by javac but not by ecj.  We
@@ -101,6 +110,10 @@ public class Main
     // We must set the bootclasspath if this isn't already done for us.
     boolean hasBootclasspath = false;
 
+    // We add the extensions directory to the end of the bootclasspath.
+    // Default to the running JVM's extensions directory.
+    String extdirs = System.getProperty ("java.ext.dirs");
+
     for (int i = 0; i < args.length; i++)
       {
         if (args[i].equals ("-bootclasspath"))
@@ -111,6 +124,13 @@ public class Main
           {
             classpathIndex = i+1;
           }
+        if (args[i].equals ("-extdirs"))
+          {
+	    extdirs = args[i+1];
+            args[i] = "-REMOVETHISARGUMENT";
+            args[i + 1] = "-REMOVETHISARGUMENT";
+	    removeCount += 2;
+          }
         else if (args[i].equals ("-sourcepath"))
           {
             sourcepath = args[i+1];
@@ -127,7 +147,7 @@ public class Main
 
     // Append the sourcepath to the classpath.
     if (sourcepath != null && classpathIndex != -1)
-      args[classpathIndex] += ":" + sourcepath;
+      args[classpathIndex] += File.pathSeparator + sourcepath;
 
     String[] strippedArgs = new String[args.length - removeCount 
                                        + (hasBootclasspath ? 0 : 2)];
@@ -135,14 +155,23 @@ public class Main
     for (int i = 0; i < args.length; i++)
       {
         if (!args[i].equals ("-REMOVETHISARGUMENT"))
-          strippedArgs[k++] = args[i];
+	  {
+	    if (!args[i].equals ("-bootclasspath"))
+	      strippedArgs[k++] = args[i];
+	    else
+	      {
+		strippedArgs[k++] = args[i++];
+		strippedArgs[k++] = args[i] + File.pathSeparator + expandExtDirs (extdirs);
+	      }
+	  }
       }
     if (! hasBootclasspath)
       {
         strippedArgs[k++] = "-bootclasspath";
-        strippedArgs[k++] = System.getProperty ("sun.boot.class.path");
+        strippedArgs[k++] = System.getProperty ("sun.boot.class.path")
+	  + File.pathSeparator + expandExtDirs (extdirs);
       }
-
+    
     Object ecjInstance = ecjConstructor.newInstance (new Object[] {
       new PrintWriter (System.out),
       new PrintWriter (System.err),
